Imports Microsoft.DirectX
'Imports Microsoft.DirectX.DirectDraw
Imports Microsoft.DirectX.Direct3D
Imports Microsoft.DirectX.Direct3D.D3DX

Public Class ExpDisp
#Region "D3D Setup Code"
    Dim ES As New ExpSettings

    Public D3Device As Microsoft.DirectX.Direct3D.Device
    Private D3Dpres As New PresentParameters
    Private D3Ddp As DisplayMode

    ' The vertex buffer that holds drawing data.
    Private DXVertexBuffer As VertexBuffer = Nothing
    '' The array of points composing each slice
    Public Pie() As VertexBuffer = Nothing
    Public Target As VertexBuffer = Nothing
    Public numPoints As Integer
    '' And the fixation cross
    Public Fixation As VertexBuffer = Nothing

    '' The array of textures composing each slice
    Public Stimuli() As Microsoft.DirectX.Direct3D.Texture = Nothing
    Public t(,) As Microsoft.DirectX.Direct3D.Texture = Nothing
    Public P(2) As Integer

    Public MaterialBlack = New Material()
    Public MaterialWhite = New Material()
    Public MaterialTarget = New Material()
    Public MaterialInvisible = New Material()

    Public fixationSize As Double = 0.005
    Public fixationThin As Double = 2

    Public Function InitializeGraphics(ByVal TargetForm As Form, ByVal FullScreen As Boolean) As Boolean

        ExpStimulusForm.Visible = True

        If FullScreen Then
            D3Ddp.Width = ES.ScreenWidth
            D3Ddp.Height = ES.ScreenHeight
            D3Ddp.RefreshRate = 60
            ''32-bit
            D3Ddp.Format = Format.A8R8G8B8
        Else
            ''If it's not fullscreen, use the current display mode
            D3Ddp = Microsoft.DirectX.Direct3D.Manager.Adapters.Default.CurrentDisplayMode
        End If

        ''Initialize some stuff for the Presentation parameters

        'Dim D3Dpres As New PresentParameters
        D3Dpres.BackBufferCount = 1
        'D3Dpres.PresentFlag = PresentFlag.LockableBackBuffer
        D3Dpres.BackBufferFormat = D3Ddp.Format
        D3Dpres.BackBufferWidth = D3Ddp.Width
        D3Dpres.BackBufferHeight = D3Ddp.Height
        D3Dpres.SwapEffect = SwapEffect.Discard
        ''Present the scene immediately
        D3Dpres.PresentationInterval = PresentInterval.One

        D3Dpres.EnableAutoDepthStencil = True    ' Depth stencil on.
        D3Dpres.AutoDepthStencilFormat = DepthFormat.D16

        If FullScreen Then
            D3Dpres.Windowed = False
        Else
            D3Dpres.Windowed = True
        End If

        ' Best: Hardware device and hardware vertex processing.
        Try
            D3Device = New Device(0, DeviceType.Hardware, TargetForm.Handle, _
                CreateFlags.HardwareVertexProcessing, D3Dpres)
            Debug.WriteLine("Hardware, HardwareVertexProcessing")
        Catch
        End Try

        ' Good: Hardware device and software vertex processing.
        If D3Device Is Nothing Then
            Try
                D3Device = New Device(0, DeviceType.Hardware, TargetForm.Handle, _
                    CreateFlags.SoftwareVertexProcessing, D3Dpres)
                Debug.WriteLine("Hardware, SoftwareVertexProcessing")
            Catch ex As Exception
                ' If we still can't make a device, give up.
                MessageBox.Show("Error initializing Direct3D" & _
                    vbCrLf & vbCrLf & ex.Message, _
                    "Direct3D Error", MessageBoxButtons.OK)
                Return False
            End Try
        End If

        ' Turn on the Z-buffer.
        D3Device.RenderState.ZBufferEnable = True

        D3Device.RenderState.AlphaBlendEnable = True

        '' Enables blending from one texture to another
        D3Device.RenderState.AlphaBlendOperation = BlendOperation.Add
        D3Device.RenderState.SourceBlend = Blend.SourceAlpha
        D3Device.RenderState.DestinationBlend = Blend.InvSourceAlpha

        D3Device.RenderState.Lighting = True

        ' Cull triangles that are oriented counter clockwise.
        D3Device.RenderState.CullMode = Cull.None

        ' Make points bigger so they're easy to see.
        D3Device.RenderState.PointSize = 3

        ' Start in solid mode.
        D3Device.RenderState.FillMode = FillMode.Solid

        ' Create the vertex data.
        ReDim Pie(frmStart.numSectors - 1)
        numPoints = frmStart.resolution * 2

        For sector As Integer = 0 To frmStart.numSectors - 1
            CreateSectorVertex(sector)
            Pie(sector) = New VertexBuffer( _
            GetType(CustomVertex.PositionTextured), _
            numPoints, D3Device, 0, _
            CustomVertex.PositionTextured.Format, _
            Pool.Default)
            Pie(sector) = DXVertexBuffer
        Next

        CreateFixationVertex()
        Fixation = New VertexBuffer( _
        GetType(CustomVertex.PositionColored), _
        12, D3Device, 0, _
        CustomVertex.PositionColored.Format, _
        Pool.Default)
        Fixation = DXVertexBuffer

        ' Make the material.
        SetupMaterial()

        ' Make the lights.
        SetupLights()

        ' We succeeded.
        Return True
    End Function

    ' Create a Sector vertex buffer for the device.
    Public Sub CreateSectorVertex(ByVal sector)
        ' Create a buffer.
        DXVertexBuffer = New VertexBuffer( _
            GetType(CustomVertex.PositionTextured), _
            numPoints, D3Device, 0, _
            CustomVertex.PositionColored.Format, _
            Pool.Default)

        ' Lock the vertex buffer. 
        ' Lock returns an array of positionTextured objects
        Dim vertices As CustomVertex.PositionTextured() = _
            CType(DXVertexBuffer.Lock(0, 0),  _
                CustomVertex.PositionTextured())

        Dim startAngle(frmStart.numSectors - 1), endAngle(frmStart.numSectors - 1) As Double

        Dim slice(frmStart.numSectors - 1, frmStart.resolution * 2 - 1) As Point
        Dim polygon(frmStart.resolution * 2 - 1) As Point
        Dim x, y As Double

        startAngle(sector) = (-90 + frmStart.clearance + 360 / frmStart.numSectors * sector) * Math.PI / 180
        endAngle(sector) = (-90 + 360 / frmStart.numSectors - frmStart.clearance + 360 / frmStart.numSectors * sector) * Math.PI / 180
        For j As Integer = 0 To frmStart.resolution - 1

            x = ES.ScreenWidth / 2 + frmStart.minDiameter * Math.Cos(startAngle(sector) + (endAngle(sector) - startAngle(sector)) * j / (frmStart.resolution - 1))
            y = ES.ScreenHeight / 2 + frmStart.minDiameter * Math.Sin(startAngle(sector) + (endAngle(sector) - startAngle(sector)) * j / (frmStart.resolution - 1))
            vertices(j * 2) = New CustomVertex.PositionTextured(x, y, 0, 0, j / frmStart.resolution)

            x = ES.ScreenWidth / 2 + frmStart.maxDiameter * Math.Cos(startAngle(sector) + (endAngle(sector) - startAngle(sector)) * j / (frmStart.resolution - 1))
            y = ES.ScreenHeight / 2 + frmStart.maxDiameter * Math.Sin(startAngle(sector) + (endAngle(sector) - startAngle(sector)) * j / (frmStart.resolution - 1))
            vertices(j * 2 + 1) = New CustomVertex.PositionTextured(x, y, 0, 1, j / frmStart.resolution)
        Next

        DXVertexBuffer.Unlock()
    End Sub

    Public Sub CreateFixationVertex()
        DXVertexBuffer = New VertexBuffer( _
            GetType(CustomVertex.PositionColored), _
            12, D3Device, 0, _
            CustomVertex.PositionColored.Format, _
            Pool.Default)

        ' Lock the vertex buffer. 
        ' Lock returns an array of positionColored objects
        Dim vertices As CustomVertex.PositionColored() = _
            CType(DXVertexBuffer.Lock(0, 0),  _
                CustomVertex.PositionColored())

        Dim halfLength As Integer = fixationSize * ES.ScreenWidth
        Dim halfThick As Integer = halfLength / fixationThin
        Dim offset(1, 11) As Integer

        '' This routine sets up the coordinates of the points of the fixation cross as one object...
        For v As Integer = 0 To 4
            offset(0, v) += halfThick
        Next
        offset(0, 11) += halfThick
        For v As Integer = 5 To 10
            offset(0, v) += -halfThick
        Next
        For v As Integer = 1 To 2
            offset(0, v) += halfLength
        Next
        For v As Integer = 7 To 8
            offset(0, v) += -halfLength
        Next

        For v As Integer = 0 To 1
            offset(1, v) += -halfThick
        Next
        For v As Integer = 8 To 11
            offset(1, v) += -halfThick
        Next
        For v As Integer = 2 To 7
            offset(1, v) += halfThick
        Next
        For v As Integer = 10 To 11
            offset(1, v) += -halfLength
        Next
        For v As Integer = 4 To 5
            offset(1, v) += halfLength
        Next

        For v As Integer = 0 To 11
            vertices(v) = New CustomVertex.PositionColored(ES.ScreenWidth / 2 + offset(0, v), ES.ScreenHeight / 2 + offset(1, v), 0, Color.White.ToArgb)
        Next

        DXVertexBuffer.Unlock()
    End Sub

    Private Sub SetupMaterial()

        MaterialInvisible.Ambient = Color.Gray
        MaterialInvisible.Diffuse = Color.Gray
        MaterialInvisible.Specular = Color.Gray
        MaterialInvisible.Emissive = Color.Gray
        MaterialInvisible.SpecularSharpness = 100

        MaterialBlack.Ambient = Color.Black
        MaterialBlack.Diffuse = Color.Black
        MaterialBlack.Specular = Color.Black
        MaterialBlack.Emissive = Color.Black
        MaterialBlack.SpecularSharpness = 100

        MaterialWhite.Ambient = Color.White
        MaterialWhite.Diffuse = Color.White
        MaterialWhite.Specular = Color.White
        MaterialWhite.Emissive = Color.White
        MaterialWhite.SpecularSharpness = 100
    End Sub

    ' Make the lights.
    Private Sub SetupLights()
        ' Make a light.
        D3Device.Lights(0).Type = LightType.Directional
        D3Device.Lights(0).Diffuse = Color.White
        D3Device.Lights(0).Ambient = Color.White
        D3Device.Lights(0).Direction = New Vector3(0, -1, 0)
        D3Device.Lights(0).Enabled = True

        D3Device.Lights(1).Type = LightType.Directional
        D3Device.Lights(1).Diffuse = Color.White
        D3Device.Lights(1).Ambient = Color.White
        D3Device.Lights(1).Direction = New Vector3(1, -1, 1)
        D3Device.Lights(1).Enabled = True

        ' Add some ambient light.
        D3Device.RenderState.Ambient = Color.Gray
    End Sub
#End Region ' D3D Setup Code

#Region "Prepare stimuli"
    Function getstimdir()
        Return (Application.StartupPath & "\stimuli")
    End Function

    Function getfn(ByVal j, ByVal k) As String
        Return (getstimdir() & "\grating_Freq" & j & "_Orient" & k & ".jpg")
    End Function

    Public Sub PrepareTextures()
        '' Prepare the stored textures
        ReDim t(frmStart.Par(0, 0) - 1, frmStart.Par(1, 0) - 1)
        '' Prepare the textures to be shown
        ReDim Stimuli(frmStart.numSectors - 1)

        For j As Integer = 0 To frmStart.Par(0, 0) - 1
            '' Countdown of stimulus preparation
            D3Device.BeginScene()
            ClearScreen()
            frmStart.DrawInstructions()
            frmStart.D3DInstFont.DrawText(Nothing, "PREPARING STIMULI!", frmStart.instSize * 2, frmStart.instSize * 33, Color.Green)
            frmStart.D3DInstFont.DrawText(Nothing, frmStart.Par(0, 0) - j, frmStart.instSize * 19, frmStart.instSize * 33, Color.Green)
            D3Device.EndScene()
            D3Device.Present()

            For k As Integer = 0 To frmStart.Par(1, 0) - 1
                t(j, k) = Microsoft.DirectX.Direct3D.TextureLoader.FromFile(D3Device, getfn(j, k))
            Next
        Next

    End Sub
#End Region

#Region "D3D Drawing Code"
    Public Sub DrawFixation(ByVal Letter As String)
        If Letter = "" Then
            D3Device.SetStreamSource(0, Fixation, 0)
            D3Device.SetTexture(0, Nothing)

            D3Device.VertexFormat = CustomVertex.PositionColored.Format

            'Draw the primitives in the data stream.
            D3Device.DrawPrimitives(PrimitiveType.TriangleFan, 0, 10)
        Else
            frmStart.D3DFixFont.DrawText(Nothing, Letter, ES.ScreenWidth / 2 - (frmStart.fixateSize / 2) * (6 / 8), ES.ScreenHeight / 2 - (frmStart.fixateSize / 2) * (11 / 8), Color.White)
        End If
    End Sub
    ''Clear screen
    Public Sub ClearScreen()
        ' Clear the back buffer and the Z-buffer.
        D3Device.Clear(ClearFlags.Target Or ClearFlags.ZBuffer, Color.Gray, 1, 0)
    End Sub

    Public Sub RenderSectors(ByVal trialI, ByVal sectorPars, ByVal TP, ByVal alpha, ByVal Letter)
        '' Clear the back buffer.
        '' Remove following line to disable clear screen to test alpha blending
        ClearScreen()

        '' Make a scene.
        D3Device.BeginScene()

        '' Setup the world, view, and projection matrices.
        SetupMatrices()

        DrawFixation(Letter)

        For sector As Integer = 0 To frmStart.numSectors - 1

            ' Set the device's data stream source (the vertex buffer).
            D3Device.SetStreamSource(0, Pie(sector), 0)

            ' Tell the device the format of the vertices.
            D3Device.VertexFormat = CustomVertex.PositionTextured.Format

            '' Set the texture depending on the parameters
            For i As Integer = 0 To frmStart.parNum - 1
                P(i) = Int(sectorPars(trialI, TP, sector, i, 0) * frmStart.Par(i, 0))
            Next

            '' Alphamap: take the alpha from the alphamap, we don't care about the color
            D3Device.SetTextureStageState(0, TextureStageStates.AlphaOperation, TextureOperation.SelectArg1)
            D3Device.SetTextureStageState(0, TextureStageStates.AlphaArgument1, TextureArgument.TextureColor)

            '' Texture: take the color from the texture, take the alpha from the previous stage
            D3Device.SetTextureStageState(0, TextureStageStates.ColorOperation, TextureOperation.SelectArg1)
            D3Device.SetTextureStageState(0, TextureStageStates.ColorArgument1, TextureArgument.TextureColor)
            D3Device.SetTextureStageState(0, TextureStageStates.AlphaOperation, TextureOperation.SelectArg1)
            D3Device.SetTextureStageState(0, TextureStageStates.AlphaArgument1, TextureArgument.Current)

            '' All stimuli are meshes (but of different colour)
            D3Device.SetTexture(0, t(P(0), P(1)))

            '' Set up a material for the relevant sector
            MaterialTarget.Ambient = Color.FromArgb(alpha(sector), 255, 255, 255)
            MaterialTarget.Diffuse = Color.FromArgb(alpha(sector), 255, 255, 255)
            MaterialTarget.Specular = Color.FromArgb(alpha(sector), 255, 255, 255)
            MaterialTarget.Emissive = Color.FromArgb(alpha(sector), 255, 255, 255)
            MaterialTarget.SpecularSharpness = 100

            D3Device.Material = MaterialTarget

            ' Draw the primitives in the data stream.
            D3Device.DrawPrimitives(PrimitiveType.TriangleStrip, 0, frmStart.resolution * 2 - 2)
        Next

        ' End the scene and display.
        D3Device.EndScene()
        D3Device.Present()
    End Sub

    ' Setup the world, view, and projection matrices.
    Private Sub SetupMatrices()
        ' World Matrix:
        D3Device.Transform.World = Matrix.Scaling(New Vector3(ES.ScreenHeight / ES.ScreenWidth, 1, 1)) ' = Matrix.Identity

        ' View Matrix:
        ' This is defined by giving:
        '       An eye point
        '       A point to look at
        '       An "up" direction

        D3Device.Transform.View = Matrix.LookAtLH( _
            New Vector3(ES.ScreenHeight / 2, ES.ScreenHeight / 2, 1000), _
            New Vector3(ES.ScreenHeight / 2, ES.ScreenHeight / 2, 0), _
            New Vector3(0, -1, 0))

        ' Projection Matrix:
        ' Perspective transformation defined by:
        '       Field of view           Pi / 4
        '       Aspect ratio            1
        '       Near clipping plane     Z = 0
        '       Far clipping plane      Z = 100
        D3Device.Transform.Projection = _
            Matrix.PerspectiveFovLH(Math.PI / 4, 1, 0, 1000)
    End Sub
#End Region
End Class
