''This program is intended to test the encoding of configuration in VSTM.
''Copyright (C) 2006 Alejandro Vicente Grabovetsky

''This program is free software; you can redistribute it and/or
''modify it under the terms of the GNU General Public License
''as published by the Free Software Foundation; either version 2
''of the License, or (at your Targetretion) any later version.

''This program is distributed in the hope that it will be useful,
''but WITHOUT ANY WARRANTY; without even the implied warranty of
''MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
''GNU General Public License for more details.

''You should have received a copy of the GNU General Public License
''along with this program; if not, write to the Free Software
''Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

''If you wish to contact the author, email alex.vice.grab@gmail.com

''Notes about program function:
''This program NEEDS the file prefpres.dll to be present in C:/Windows/system/ to
''make the timing accurate.

Imports System.Text
Imports System.IO

Public Class frmStart
    '' Requires 394 scans at TR=2, 13.1 mins
    '' To check: 
    ''   eccentricity and size of stimuli
    ''   ignore central position

    Dim ED As New ExpDisp
    Dim ES As New ExpSettings
    Dim ET As New ExpTimingAndResponses
    Dim TL As AttentionTrialList


    Public RTcomms As New CBURTcommunications ' must be only one of these around!

#Region "Declare all constants"
    '' TrialList settings
    Public Block As Integer '' Number of blocks (type of cue varies in AB fashion)
    Public Adaptive As Integer
    Public Duration_Att As Integer = 3
    Public trialNumber As Integer

    '' Trial settings for this experiment:
    '' Number of slices present
    Public numSectors As Integer
    Public parNum As Integer = 2
    '' Image parameters: resolution of each slice
    Public resolution As Integer
    Public minDiameter As Integer     '' Minimum diameter of sector
    Public maxDiameter As Integer     '' Maximum diameter of sector
    Public clearance As Integer       '' Angle clearnace between sectors

    '' Temporal parameters
    Public StillInt As Integer
    Public IntStep As Integer
    Public DelayInt As Integer
    Public RespInt As Integer
    Public StopInt As Integer
    '' Letters
    Public LetterNum As Integer

    '' Constant parameters
    Public Repetitions As Integer
    Public Contrasts As Integer
    Public Targets As Integer
    Public MinAlpha As Integer
    Public MaxAlpha As Integer

    '' Contrast parameters & psychometric function
    Public alpha() As Integer
    Public Slope As Double
    Public Threshold As Double
    Public MinPsy, MaxPsy, Range As Double

    Public accK As Double
    Public velK As Double
    Public posBoundForce As Double
    Public posBoundPower As Integer
    Public velBoundForce As Double
    Public velBoundPower As Integer

    Public Comb() As String

    '' The # of parameters' limits, dimensionality (circular "0" or bounded "1") and difficulty (difference between same/different trials)
    Public Par(,) As Integer
    Public corner(,) As Integer             '' Position of square templates on screen
    Public distance(,), angle(,), surface(,) As Integer     '' x,y surface parameters: indexing

    '' Trials on which there are Target stops overall... and when they occur if there are any...
    Public TargetTrials(,) As Integer

    '' Each Trial contains a number of TPs with a # of Sectors defined by a # of parameters
    '' and each parameter has values for: value (0 to max. value), speed of change (-1 to 1) and acceleration (-1 to 1)
    Public sectorPars(,,,,) As Double
    '' These can be summarised over the trial as sector averages across TPs only for position:
    Public sectorAvgs(,,) As Double

    '' The font for the instructions and feedback
    Public instSize As Integer = ES.ScreenWidth / 50
    Public InstructionFont As New Font("arial", instSize, FontStyle.Regular, GraphicsUnit.Pixel)
    Public D3DInstFont As Microsoft.DirectX.Direct3D.Font

    '' The font for the feedback signal
    Public FeedbackSize As Integer = 100
    Public FeedbackFont As New Font("arial", FeedbackSize, FontStyle.Bold, GraphicsUnit.Pixel)
    Public D3DFeedFont As Microsoft.DirectX.Direct3D.Font

    '' The font for the fixation "X" and surround
    Public fixateSize As Integer = ES.ScreenWidth / 80
    Public fixateFont As New Font("arial", fixateSize, FontStyle.Bold, GraphicsUnit.Pixel)
    Public D3DFixFont As Microsoft.DirectX.Direct3D.Font

    Public instColor As System.Drawing.Color = Color.White
#End Region

    Private Sub frmStart_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Randomize()
        'Dim fluff As Integer = GetFileContents(Application.StartupPath & "/adaptive.txt")

        ' This says Hello to the real time and provides it with our IP address, so it can find us
        ' It then waits for a response. 
        ' Put it at the start of the programme so it crashes immediately if the communication
        '  cannot be achieved.

        RTcomms.Handshake()


        'Dim inP As Double
        'Dim outP As Double
        'inP = 8
        'outP = ParameterPsy(inP)
        'MsgBox("in " & inP & " out " & outP)
    End Sub

    Public Function ParameterPsy(ByVal p)
        Dim Y As Double

        Y = 1 + ((Math.Log(255) / Math.Log(2) - 1) / (1 + Math.Exp(-(p - 2.5) / 0.4)))

        Return Y
    End Function


    Public Function GetFileContents(ByVal FullPath As String, _
Optional ByRef ErrInfo As String = "") As String
        Dim strContents As String
        Dim objReader As StreamReader
        Dim fs As New FileStream(FullPath, FileMode.Open, FileAccess.Read, FileShare.ReadWrite)
        objReader = New StreamReader(fs)
        Do
            strContents = objReader.ReadLine()
        Loop Until (objReader.EndOfStream)
        objReader.Close()
        Return strContents

    End Function
    Public Sub cmdStart_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdStart.Click
        SetConstants()

        '' WE MAY NEED TO CHANGE THIS
        trialNumber = Contrasts * Repetitions

        TL = New AttentionTrialList(trialNumber, txtOutputFilename.Text, txtSubjectCode.Text)

        ED.InitializeGraphics(ExpStimulusForm, ES.FullScreen)
        D3DInstFont = New Microsoft.DirectX.Direct3D.Font(ED.D3Device, InstructionFont)
        D3DFeedFont = New Microsoft.DirectX.Direct3D.Font(ED.D3Device, FeedbackFont)
        D3DFixFont = New Microsoft.DirectX.Direct3D.Font(ED.D3Device, fixateFont)

        TL.SetExperimentalDisplay(ED)

        '' Initialize keyboard
        TL.ET.InitKeyboard(ExpStimulusForm)
        TL.ET.DXInputKeyboard.Acquire()

        '' Set up trials with Target stops
        Dim ind As Integer
        ReDim TargetTrials(trialNumber - 1, 1)
        For s As Integer = 1 To Targets
            Do
                ind = Rnd() * trialNumber - 0.5
            Loop Until TargetTrials(ind, 0) = 0
            TargetTrials(ind, 0) = 1
            TargetTrials(ind, 1) = Rnd() * LetterNum - 0.5
        Next

        ind = 0

        If Adaptive = 0 Then
            For i As Integer = 1 To Contrasts
                For j As Integer = 0 To Repetitions - 1
                    TL.TrialList(ind).AddParameter("Block", Block)
                    TL.TrialList(ind).AddParameter("Target", TargetTrials(ind, 0))

                    If TargetTrials(ind, 0) = 1 Then
                        TL.TrialList(ind).AddParameter("Target_Pos", TargetTrials(ind, 1))
                    Else
                        TL.TrialList(ind).AddParameter("Target_Pos", "N/A")
                    End If

                    TL.TrialList(ind).AddParameter("Contrast", i)
                    TL.TrialList(ind).AddParameter("Subblock", j)
                    ind = ind + 1
                Next
            Next

            '' Randomise
            TL.RandomizeTrialOrder(True)  '' This preserves the SUBBLOCKS containing sets of all sector combinations

        ElseIf Adaptive = 1 Then
            For i As Integer = 0 To trialNumber - 1
                TL.TrialList(ind).AddParameter("Block", Block)
                TL.TrialList(ind).AddParameter("Target", TargetTrials(ind, 0))

                If TargetTrials(ind, 0) = 1 Then
                    TL.TrialList(ind).AddParameter("Target_Pos", TargetTrials(ind, 1))
                Else
                    TL.TrialList(ind).AddParameter("Target_Pos", "N/A")
                End If

                ind = ind + 1
            Next

            '' Randomise
            TL.RandomizeTrialOrder(False)  '' This preserves the SUBBLOCKS containing sets of all sector combinations

        End If


        '' The parameter ranges
        ReDim Par(parNum - 1, 1)
        '' The parameter limits, depending on how many permutations of the parameter you have
        Par(0, 0) = 30
        Par(1, 0) = 90

        '' The parameter dimensionality, whether the parameter is unbounded/circular ("0") or bounded/linear ("1")
        Par(0, 1) = 1
        Par(1, 1) = 0

        ReDim alpha(numSectors - 1)

        '' Declare a number of TPs, where TP 0 actually corresponds to the probe stimulus, decided at the end...
        ReDim sectorPars(trialNumber - 1, 60 * (StillInt + IntStep * Duration_Att) / 1000, numSectors - 1, parNum - 1, 2)
        ReDim sectorAvgs(trialNumber - 1, numSectors - 1, parNum - 1)

        ED.PrepareTextures()

        '' Flush form change and so on
        Application.DoEvents()

        '' WRITE OUT ALL THE PLANNED EVENTS? @@@

        '' Run
        TL.RunAllTrials()

        '' Finish
        DrawFinish()
        'ED.Terminate()
    End Sub

#Region "Instructions"
    Sub DrawInstructions()

        D3DInstFont.DrawText(Nothing, "A screen will appear, containing a disc made up of " & numSectors & " sectors", instSize * 2, instSize * 10, instColor)
        D3DInstFont.DrawText(Nothing, "The image in each sector will change over time", instSize * 2, instSize * 12, instColor)
        D3DInstFont.DrawText(Nothing, "A stream of letters will appear in the centre of the disc", instSize * 2, instSize * 14, instColor)
        D3DInstFont.DrawText(Nothing, "Sometimes the letter will be an X, if that's the case, respond as quickly as possible!", instSize * 2, instSize * 16, instColor)
        D3DInstFont.DrawText(Nothing, "It's VITAL that you remain FIXATED on the centre of the screen constantly!", instSize * 2, instSize * 23, Color.Yellow)

    End Sub

    Sub DrawFinish()

        ED.D3Device.BeginScene()
        ED.ClearScreen()
        If Block = 2 Then
            D3DInstFont.DrawText(Nothing, "You have finished this EXPERIMENT!", ES.ScreenWidth / 4, ES.ScreenHeight / 2 - instSize, instColor)
        Else
            D3DInstFont.DrawText(Nothing, "You have finished this MINI-BLOCK!", ES.ScreenWidth / 4, ES.ScreenHeight / 2 - instSize, instColor)
            D3DInstFont.DrawText(Nothing, "Please REST your eyes for a couple of minutes!", ES.ScreenWidth / 4, ES.ScreenHeight / 2 + instSize, instColor)
        End If
        ED.D3Device.EndScene()
        ED.D3Device.Present()

        TL.ET.GetTimer()
        TL.ET.WaitForTime(TL.ET.GetTimer() + 2000)
    End Sub
#End Region

    Sub SetConstants()

        '' Number of slices present
        numSectors = SectorsBox.Text
        clearance = ClearanceBox.Text
        minDiameter = MinDiamBox.Text
        maxDiameter = MaxDiamBox.Text
        '' Resolution of the slices and their two diameters and separation (clearance)
        resolution = ResBox.Text

        '' Parameter change parameters:
        '' Constants dictating how effective acceleration is on velocity and velocity on position
        accK = AccKBox.Text
        velK = VelKBox.Text
        '' Bounds that limit the maximum position (for bounded variables) and velocity (for all stimuli)
        posBoundForce = PBoundFBox.Text
        posBoundPower = PBoundPBox.Text
        velBoundForce = VBoundFBox.Text
        velBoundPower = VBoundPBox.Text

        '' Temporal parameters of all trials
        StillInt = StillIntBox.Text
        DelayInt = DelayIntBox.Text
        RespInt = RespIntBox.Text
        StopInt = StopIntBox.Text
        '' Letters in each trial
        LetterNum = Math.Round((StillInt + DelayInt) / RespInt)

        Repetitions = NumRepsBox.Text
        Contrasts = NumContBox.Text
        Targets = TargetCountBox.Text
        MinAlpha = MinAlphaBox.Text
        MaxAlpha = MaxAlphaBox.Text

        '' Psychometric function
        Slope = 0.4
        MinPsy = Math.Log(0.4, 2)
        MaxPsy = Math.Log(255, 2)
        Range = MaxPsy - MinPsy

        If ThresholdBox.Text = Nothing Then
            MsgBox("No Threshold chosen!")
            End
        Else
            Threshold = ThresholdBox.Text
        End If


        If BlockBox1.Checked Then
            Block = 0
        ElseIf BlockBox2.Checked Then
            Block = 1
        ElseIf BlockBox3.Checked Then
            Block = 2
        Else
            MsgBox("No block chosen!")
            End
        End If

        If AdaptiveBox1.Checked Then
            Adaptive = 0
        ElseIf AdaptiveBox2.Checked Then
            Adaptive = 1
        Else
            MsgBox("No mode chosen!")
            End
        End If
    End Sub
End Class